from jivi.oe import *
from jivi import *
from jivi.oe.constants import *

  
read_images_tab = {'image' : 'image','image-up' : 'image','image-down' : 'image-down','image-insensitive' : 'image-insensitive'}

# FILE name
   # [{ IMAGE-SIZE | IMAGE-SIZE-CHARS | IMAGE-SIZE-PIXELS }
      # width BY height
# ]
# [ FROM { X n Y n | ROW n COLUMN n }]



# { IMAGE-SIZE | IMAGE-SIZE-CHARS | IMAGE-SIZE-PIXELS }
   # width BY height
# [ FROM { X n Y n | ROW n COLUMN n }]


def read_image(ar,i):
	new_ar = ar[:i]
	ar = ar[i+1:]
	t = {'file' : 0,'from' : []}
	
	
	kw = full_keyword(ar[0])
	if kw == 'file':
		t['file'] = ar[1]
		ar = ar[2:]
		if not ar: return (new_ar + ar,t)
		
		kw = full_keyword(ar[0])
	
	
	
	if kw.find('size') != -1:
		t['w'] = wtp(ar[1],kw.find('pix'))
		t['h'] = htp(ar[3],kw.find('pix'))
		ar = ar[4:]
		if not ar: return (new_ar + ar,t)
		kw = full_keyword(ar[0])
		
	if kw == 'from':
		t['from'] = ar[:4]
		ar = ar[4:]
	
	return (new_ar + ar,t)
	
		
	
def read_images(ob,ar):

	for i,x in enumerate(ar):
		kw = full_keyword(x)
		if kw in read_images_tab:
			ar,image = read_image(ar,i)
			ob.set_attr(read_images_tab[kw],image)
			return read_images(ob,ar)
	return ar
def read_size(ob,ar):
	size_chars = ['size','size-chars']

	for i,x in enumerate(ar):
		kw = full_keyword(x)
		found = 0
		w = 0
		h = 0
		
		if kw in size_chars:
			w = wtp(float(ar[i+1]))
			h = htp(float(ar[i+3]))
			found = 1
		elif kw == 'size-pixels':
			w = (float(ar[i+1]))
			h = (float(ar[i+3]))
			found = 1
		if found:
			ob.set_attr('width',round(w,2))
			ob.set_attr('height',round(h,2))
			
			ar = ar[:i] + ar[i+4:]
			break
	return ar
			

def read_query(ob,ar):
	
	ob.set_attr('query',ar)
	return ar



class view_as:
	all_types = {k : k.replace('-','_') for k in 'fill-in,combo-box,editor,toggle-box,radio-set,text,selection-list,slider'.split(',')}

	def __init__(self,ob,ar):
		self.tab = {}
		self.ob = ob
		self.ar = ar
		self.type = ar[0]
		self.ar = ar[1:]
		if not self.type in self.all_types:
			print(self.type)
			print(self.ar)
			
		self.ar = read_size(ob,self.ar)
		getattr(self,self.all_types[self.type])()
	def fill_in(self):
		pass
	def combo_box(self):
		pass
	def editor(self):
		pass
	def toggle_box(self):
		pass
	def radio_set(self):
		pass
	def text(self):
		pass
	def selection_list(self):
		pass
	def slider(self):
		pass

	def val(self):
		return dict(type=self.type,rest=self.ar)
	
	
to_html_button_format = """<button style="position: absolute; left: {left}px; top: {top}px;height: {height}px; width: {width}px;" title = {label}></button>"""
def to_html_button(tab):

	t = {}
	for a in 'label,left,top,height,width'.split(','):
		t[a] = tab[a] if a in tab else ''
	return to_html_button_format.format(**t)
	
to_html_fill_in_format = """<input type = "text" style="position: absolute; left: {left}px; top: {top}px;height: {height}px; width: {width}px;" value = {label}>"""
def to_html_fill_in(tab):

	t = {}
	for a in 'label,left,top,height,width'.split(','):
		t[a] = tab[a] if a in tab else ''
	if not t['label']: t['label'] = '""'
	return to_html_fill_in_format.format(**t)

class Widget:
	has_datatype = {k : 1 for k in ['parameter','variable']}
	def __init__(self,ar,left=0,top=0,extra=[]):
		
		self.original = ar[:]
		self.ar = ar
		self.tab = dict(left=left,top=top)
		self.view_as = 0
		self.type = 0
		
		self.strip_keyword('def')
		self.succes = self.do()
		self.tab['rest'] = self.ar
		
		
		
	def set_size(self):
		read_size(self)
	@property
	def html(self):
		if not self.succes: return ''
		
		if self.type == 'button':
			return to_html_button(self.tab)
			
		if self.view_as and 'type' in self.view_as and self.view_as['type'] == 'fill-in':

			return to_html_fill_in(self.tab)
		return ""
		
		
	def do(self):

		if not self.set_type_and_name():
			self.Error('failed to set set_type_and_name')
			print(self.ar)
			return 0
		
		if self.type in self.has_datatype:
			if not self.set_datatype():
				return 0
		
		if self.type == 'temp-table':
			self.read_tt()
			return
		

		self.set_query()
		self.set_view_as()
		self.ar = read_images(self,self.ar)
		
		self.set_init()
		self.set_attr('format',self.read_one('format'))
		self.set_singles()
		
		self.ar = read_size(self,self.ar)
		self.set_vals()
		self.set_images()
		
		
		return 1
	def read_tt(self):
		self.ar = []
	def set_query(self):
		for i,a in enumerate(self.ar):
			if full_keyword(a) == 'query':
				read_query(self,self.ar[i+1:])

				self.ar = self.ar[:i]
				break
		
	def set_images(self):
		pass
	def set_vals(self):
		for i,a in enumerate(self.ar):
			kw = full_keyword(a)
			if kw in input_keywords:
				self.set_attr(kw,self.ar[i+1])
				self.ar = self.ar[:i] + self.ar[i+2:]
				return self.set_vals()
	def set_singles(self):
		
		for i,a in enumerate(self.ar):
			kw = full_keyword(a)
			if kw in keyword_single:
				self.set_attr(kw,1)
				self.ar = self.ar[:i] + self.ar[i+1:]
				return self.set_singles()
	
	def set_view_as(self):
		for i,a in enumerate(self.ar):
			if full_keyword(a) == 'view-as':
				tmpx = self
				self.view_as = view_as(tmpx,self.ar[i+1:]).val()
				
				self.ar = self.ar[:i]
				break

		

		
	def set_init(self):
		self.init = self.read_one('init')
		return self.init
	def make_tab(self):
		self.left = self.ar
		return {k : getattr(self,k) for k in self.new_tab_attr}
	

		
	def read_one(self,kw):
		kw = full_keyword(kw)
		for i,a in enumerate(self.ar):
			if full_keyword(a) == kw:
				val = self.ar[i+1]
				self.ar = self.ar[:i] + self.ar[i+2:]
				return val
	
	def set_datatype(self):
		return self.set_attr('datatype',self.read_one('as'))

	
	
	
	def set_attr(self,k,v):
		setattr(self,k,v)
		self.tab[k] = v
		return v
	
	def set_type_and_name(self):
		for i,a in enumerate(self.ar):
			if a in define_types:
				self.set_attr('type',define_types[a])
				self.set_attr('name',self.ar[i+1])
				self.ar = self.ar[:i] + self.ar[i+2:]
				return 1
		
	def strip_keyword(self,kw):
		kw = full_keyword(kw)
		for i,ar_kw in enumerate(self.ar):
			if full_keyword(ar_kw) == kw:
				self.ar = self.ar[:i] + self.ar[i+1:]
				break
				
	def Error(self,s):
		print(self.original)
		print(s)



from jivi.fs import *
from os import system
from time import time
from jivi.oe.constants import *

def is_decimal(x):
	try:
		s = float(x)
		return 1
	except:
		return 0
		
zoek_ar = [(a,b,len(a)) for a,b in [('/*',1),('*/',-1),('"',2),("'",3),('{',5),('}',-5),('&glob',4),('&scop',4)]]
is_string = {3 : 1,2 : 1}
is_comment = {1 : 1, -1 : 1}
is_include = {5 : 1, -5 : 1}
is_scope = 4
string_appenders = {':%s' % k : 1 for k in 'r,l,c,t,u'.split(',')}
str_numbers = {str(i) : 1 for i in range(10)}

replace_with = ' R'
replace_with_len = len(replace_with)

do_rep = {k : 1 for k in 'variable,define'.split(',')}

class Reader:
	funcs = 'str_com,make_replaced,do_connected,make_replaced_two,make_ar,set_replaced_txt,make_new_commands_ar,set_indent'.split(',')

	times = {k : 0 for k in funcs}

	def __init__(self,fp=None,txt=None):
		self.errors = []
		if fp is not None: 	
			if not self.set_fp(fp):
				return
			
		if txt is not None: self.set_txt(txt)
		for k in self.funcs:
			ss = time()
			getattr(self,k)()
			self.times[k] += time() - ss

	def set_indent(self):
		self.indent = []

	def old_write_o(self):
		self.replaced_txt = [self.txt_o[start:stop] for start,stop,x in self.replaced]
		self.commands_txt = []
		


		for i,c in enumerate(self.commands):

			self.commands_txt.append(c[:])
			for j,cw in enumerate(self.commands_txt[i]):
				
				if isinstance(cw,int):
					self.commands_txt[i][j] = self.replaced_txt[cw]
					
					 
		File.jwrite("o\\" + self.name + ".json",self.commands_txt)
	
	
	def set_replaced_txt(self):
		self.replaced_txt = []
		for start,stop,x in self.replaced:
			txt = self.txt_o[start:stop].strip()
			if txt.endswith(':u'):
				txt = txt[:-2]
			
			self.replaced_txt.append(txt)
			

		
	def make_new_commands_ar(self):
		
		
		
		self.commands_txt = []
		
		new_commands = []

		for i,c in enumerate(self.commands):
			new_commands_ar = []
			replaced_last_one = 0
			found_special = 0
			for j,cw in enumerate(c):
				#found_special = found_special or cw == 'radio-buttons'
				if isinstance(cw,int):
					
					if replaced_last_one:
						new_commands_ar[-1] += self.replaced_txt[cw]
					else:
						new_commands_ar.append(self.replaced_txt[cw])
					replaced_last_one = 1
				
				elif cw.find(',') != -1 or (replaced_last_one and is_decimal(cw)):
					if replaced_last_one:
						new_commands_ar[-1] += cw
					else:
						new_commands_ar.append(cw)
					replaced_last_one = 1
	
					
				elif cw:
					replaced_last_one = 0
					if cw in alias and alias[cw] in do_rep:
						new_commands_ar.append(alias[cw])
					else:
						new_commands_ar.append(cw)
			if new_commands_ar: new_commands.append(new_commands_ar)
			if found_special:
				print(new_commands_ar)
				exit()
		
		self.new_commands = new_commands

	def write_o(self):
		self.replaced_txt = [self.txt_o[start:stop] for start,stop,x in self.replaced]
		self.commands_txt = []
		
		new_commands = []

		for i,c in enumerate(self.commands):
			new_commands_ar = []
			for j,cw in enumerate(c):
				if isinstance(cw,int):
					new_commands_ar.append(self.replaced_txt[cw])
				elif cw:
					new_commands_ar.append(cw)
			if new_commands_ar: new_commands.append(new_commands_ar)
			
		
			
		File.jwrite("o\\" + self.base,new_commands)

		
	def make_ar(self):
		self.txt = self.txt.replace("\n"," ").replace('R',' R ')
		while self.txt.find('  ') != -1:
			self.txt = self.txt.replace('  ', ' ')
		#self.commands = [c.strip().split(' ') for c in rec.sub(' ',self.txt.replace("\n"," ")).replace('R',' R ').split('.')]
		self.commands = [c.strip().split(' ') for c in self.txt.split('.')]
		#self.commands = [[w.strip() for w in c.split(' ') if w.strip()] for c in self.txt.replace("\n", " ").replace('R',' R ').split('.')]
		self.replaced_good_index = 0
		for i,c in enumerate(self.commands):
			for j,cw in enumerate(c):
				if self.commands[i][j] == 'R':
					self.commands[i][j] = self.replaced_good_index
					self.replaced_good_index += 1




	def set_fp(self,fp):
		self.fp 	= File.fp(fp)
		self.name 	= File.name(fp)
		self.base 	= File.base(fp)
		self.ex		= File.ex(fp)
		return self.set_txt(txt= File.all(fp))

	def filter_lines_old(self,lines):
		self.lines = []
		next_append = 0
		self.pre_defines = []
		inum = 0
		while inum < len(lines):
			l = lines[inum].lower().strip()
			if (not l) or l.startswith('&analyze'):
				inum += 1
				continue
			if l.startswith(('&scop','&glob')):
				if not l.endswith('~'):
					self.pre_defines.append(l)
					inum += 1
					continue
				line_ar = [l]
				while l.endswith('~') and inum < len(lines):
					inum += 1
					l = lines[inum].lower().strip()
					line_ar.append(l)
				self.pre_defines.append(" ".join(line_ar))
				inum += 1
				continue
			if l.startswith('&'):
				if not l.endswith('~'):
					self.lines.append(l)
					inum += 1
					continue
				line_ar = [l]
				while l.endswith('~') and inum < len(lines):
					inum += 1
					l = lines[inum].lower().strip()
					line_ar.append(l)
				self.lines.append(" ".join(line_ar))
				inum += 1
				continue
			else:
				self.lines.append(l)
				inum += 1
				continue
	def filter_linesx(self,lines):
		self.lines = []
		next_append = 0
		self.pre_defines = []
		inum = 0
		ll = len(lines) - 1
		while inum < ll:
			inum += 1
			l = lines[inum].lower().strip()
			if (not l) or l.startswith('&analyze'):
				continue
			if l.startswith('&'):
				if not l.endswith('~'):
					self.lines.append(l)
					continue
				line_ar = [l]
				while l.endswith('~') and inum < len(lines):
					inum += 1
					l = lines[inum].lower().strip()
					line_ar.append(l)
				self.lines.append(" ".join(line_ar))
				continue
			else:
				self.lines.append(l)
				continue
	def filter_lines(self,lines):
		self.lines = []
		next_append = 0
		self.pre_defines = []
		inum = -1
		ll = len(lines) - 1
		while inum < ll:
			inum += 1
			l = lines[inum].lower().strip()
			if (not l) or l.startswith('&analyze'):
				continue
			if l.endswith('~'):
				line_ar = [l]
				while l.endswith('~') and inum < len(lines):
					inum += 1
					l = lines[inum].lower().strip()
					line_ar.append(l)
				self.lines.append(" ".join(line_ar))
				continue
			else:
				self.lines.append(l)
				continue

	def set_txt(self,txt):
		if not txt: return False
		txt = txt.replace("\t"," ")
		lines 				= list(txt.splitlines())
		self.filter_lines(lines)
		self.txt 			= "\n".join(self.lines)
		self.txt_o			= self.txt[:]
		self.txt_o_len		= len(self.txt_o)
		return 1

	def str_com(self):

		count = 0
		start_index = 0
		ar = []
		for needle,value,la in zoek_ar:
			index = self.txt.find(needle)
			while index != -1:
				ar.append((index,value))
				index = self.txt.find(needle,index+la)
		ar.sort()
		inum = 0
		lar = len(ar)
		larminone = lar - 1
		temp_ar = []
		last_stop = 0
		self.replaced = []
		while inum < lar:
			start_inum = inum
			start_index,val = ar[inum]
			if val == is_scope:
				
				if (not start_index) or self.txt[start_index - 1:start_index] == "\n":
					stop = self.txt.find("\n",start_index)
					if stop == -1: stop = len(self.txt)
					self.pre_defines.append(self.txt[start_index:stop])
					start = start_index
					self.replaced.append((start,stop,0))
					if start != last_stop:
						temp_ar.append(self.txt[last_stop:start])
					last_stop = stop

				else:
					print(start_index)
					stop = self.txt.find("\n",start_index)
					if stop == -1: stop = len(self.txt)
					print(self.txt[start_index:stop])
					print(self.fp)
					exit()
					

			elif val in is_string:
				inum += 1
				string_val = val
				string_found = 0
				while inum < larminone:
					if (ar[inum][1] != string_val):
						inum += 1
					elif (ar[inum + 1][1] == string_val) and (ar[inum + 1][0] == (ar[inum][0] + 1)):
						inum += 2
					else:
						string_found = ar[inum][0]
						break
				if (not string_found) and (inum < lar) and (ar[inum][1] == string_val):
					string_found = ar[inum][0]
				if string_found and self.txt[string_found + 1:string_found + 3] in string_appenders:
					string_found += 2
					while self.txt[string_found+1] in str_numbers:
						string_found += 1
				if not string_found:
					self.add_error_at_pos("do_str_c_pre : not string_found",start_index,func='add_all_strings')
					string_found = start_index
					inum = start_inum
				start = start_index
				stop = string_found + 1
				if start != last_stop:
					temp_ar.append(self.txt[last_stop:start])
				temp_ar.append(replace_with)
				self.replaced.append((start,stop,replace_with_len))
				last_stop = stop
			elif val in is_include:
				count = val
				inum += 1
				found = 0
				while inum < lar:
					index,val = ar[inum]
					if val in is_include:
						count += val
						if count < 0: print("Error !")
						if not count:
							start = start_index
							stop = index + 1
							if start != last_stop:
								temp_ar.append(self.txt[last_stop:start])
							temp_ar.append(replace_with)
							self.replaced.append((start,stop,replace_with_len))
							last_stop = stop
							found = 1
							break
					inum += 1
				if not found:
					self.add_error_at_pos("do_str_c_pre : is_include not found! started at line ",start_index)
			elif val in is_comment:
				last_index_plus_one = 0
				count = val
				inum += 1
				found_comment = 0
				while inum < lar:
					index,val = ar[inum]
					if val in is_comment and last_index_plus_one != index:
						count += val
						if count < 0: print("Error !")
						if not count:
							start = start_index
							stop = index + 2
							self.replaced.append((start,stop,0))
							if start != last_stop:
								temp_ar.append(self.txt[last_stop:start])
							last_stop = stop
							found_comment = 1
							break
						last_index_plus_one = index + 1
					inum += 1
				if not found_comment:
					self.add_error_at_pos("do_str_c_pre : is_comment not found_comment! started at line ",start_index)
					
			while inum < lar and (ar[inum][0] < last_stop):
				inum += 1
			
			
		temp_ar.append(self.txt[last_stop:])
		self.txt = "".join(temp_ar)

	def make_replaced_two(self):
		self.replaced.sort()
		nr = []
		last_stop = -1
		last_start = -1
		for start,stop,l in self.replaced:
			if last_start < start < last_stop:
				last_stop = max(last_stop,stop)
				nr[-1] = (last_start,last_stop,1)
			else:
				nr.append((start,stop,l))
				last_stop = stop
				last_start = start
		self.replaced = nr

	def make_replaced(self):
		self.replaced.sort()
		nr = []
		last_stop = -1
		last_start = -1
		for start,stop,l in self.replaced:
			if last_start < start < last_stop:
				last_stop = max(last_stop,stop)
				nr[-1] = (last_start,last_stop,1)
			else:
				nr.append((start,stop,l))
				last_stop = stop
				last_start = start
		self.replaced = nr
		self.replaced_helper = []
		n = 0
		for start,stop,l in self.replaced:
			lengte = stop - start - l
			self.replaced_helper.append((start - n,n))
			n += (stop - start) - l
		self.replaced_helper.append((self.txt_o_len + 10,n))

	def get_real_pos_add_ar(self,ar):
		iar = 0
		lar = len(ar)
		if not lar: return ar
		for start,n in self.replaced_helper:
			if start <= ar[iar][0]: continue
			while iar < lar and ar[iar][0] < start:
				ar[iar] = (ar[iar][0] + n,ar[iar][1] + n, ar[iar][2])
				iar += 1
			if iar == lar: break
		return ar

	def do_connected(self):
		temp_ar = []
		self.txt = self.txt.replace(':','.').replace("\n", " ")
		a = self.txt.rfind('.')
		next_dot = 0
		while a != -1:
			bluh = a
			if self.txt[a+1:a+2].strip():
				stop = self.txt.find(' ',a)
				if stop == -1:
					stop = len(self.txt)
				elif stop == next_dot + 1:
					stop -= 1
				bluh = self.txt.rfind(' ',0,a) + 1
				temp_ar.append((bluh,stop))
			next_dot = a
			a = self.txt.rfind('.',0,bluh-1)
		s_ar = []
		last_stop = 0
		other_ar = []
		temp_replaced = []
		for start,stop in reversed(temp_ar):
			if start != last_stop:
				s_ar.append(self.txt[last_stop:start])
			s_ar.append(replace_with)
			temp_replaced.append((start,stop,replace_with_len))
			last_stop = stop
		s_ar.append(self.txt[last_stop:])
		self.txt = "".join(s_ar)
		self.replaced = [a for a in self.replaced if a[2]] + self.get_real_pos_add_ar(temp_replaced)



	def get_line(self,pos):
		return self.txt_o.count("\n",0,pos)

	def add_error_at_pos(self,s,pos,*a,**b):
		l = self.get_line(pos)
		if l != None:
			return self.add_error(s + "\nat line {l}".format(l=l+1),*a,**b)
		return self.add_error(s + "\n at pos " + str(pos),*a,**b)

	def add_error(self,s,func=0):
		self.errors.append(s)
		
		
		

	def write_error(self,error_name,s,add=1):
		fp = "out\\" + self.name + "__" + error_name + self.ex
		with open(fp,"w") as f:
			f.write(s)
		if add:
			self.errors.append(fp)
		return fp
		
		
	def test_replaced(self):
		if len(self.replaced) != self.replaced_good_index:
			self.write_error("test_replaced","\n".join([self.txt_o[start:stop] for start,stop,x in self.replaced]))
			self.errors.append("Count R = %d" % self.txt.count('R'))
			self.errors.append("len_replacers = %d" % len(self.replaced))
	
	def test(self):
		self.test_replaced()
		if not self.errors: return
		self.test_write()
		
		
		
	def test_write(self):
		if self.errors:
			with open("out\\" + self.name + "_errors.txt","w") as f:
				f.write("\n".join([self.fp] + self.errors))
		self.replaced.sort()
		original = self.txt
		for start,stop,l in self.replaced:
			if not l: continue
			original = original.replace('R',self.txt_o[start:stop],1)
			
		
		with open("out\\" + self.base,"w") as f:
			f.write(self.txt)
		self.write_error("original",original)
		self.write_error("txt_o",self.txt_o)
		
		


from jivi.fs import *
from jivi.oe import *
from jivi.oe.Reader import Reader as oeReader
from jivi.oe.ArHelper import ArHelper
from jivi.oe.StatementReader import StatementReader as SR
from jivi.oe.constants import *

class Window:
	steps = ['set_oeReader','set_window','set_widgets','set_frames','read_frames']
	def __init__(self,environment,fp):
		self.environment = environment
		self.fp = File.fp(fp).lower()
		self.window = {}
		self.frames = {}
		self.widgets = {}
		for a in Window.steps:
			if not getattr(self,a)():
				print('failed ' + a)
				break
		
		
	def set_oeReader(self):
		self.oeReader = oeReader(self.fp)
		if not hasattr(self.oeReader,'new_commands'): return 0
		self.commands = self.oeReader.new_commands
		return 1
		
	def set_window(self):
		found = self.strip_command(['create','window'],1)
		if not found: return 0
		self.window_index = self.strip_command_last_found_index
		found = ArHelper.strip_until(found,'assign')
		if not found: return 0
		self.window = SR.assign(found)
		return self.window
			
	def set_widget(self,ar):
		for i,x in enumerate(ar):
			if x in define_types:
				ar[i] = define_types[x]
				ar[i+1] = ar[i+1].lower()
				widget = dict(type=ar[i],name=ar[i+1])
				ar=ar[i+2:]
				
				if widget['type'] == 'buffer':
					ar,p = ArHelper.strip_property(ar,'for')
					widget['table'] = self.environment.get_table_name(p)
				elif widget['type'] == 'temp-table':
					#self.read_temp_table(...) !todo
					pass
				
				widget['rest'] = ar
				self.widgets[widget['name']] = widget
				return 1
	def set_widgets(self):
		for i,c in enumerate(self.commands):
			if i > self.window_index: return 1
			if c[0] != 'define': continue
			self.set_widget(c)
	
		
	def set_frames(self):
		for name,widget in self.widgets.items():
			if not widget['type'] == 'frame': continue
			self.frames[name] = widget
	

	def read_frames(self):
		for framname,frame in self.frames.items():
			items = []
			with_statement = []
			
	def strip_command(self,ar,fullblock=1):
		for i,c in enumerate(self.commands):
			for j,cl in enumerate(c):
				if cl == ar[0] and ArHelper.same(c[j:],ar):
					ret_val = c[j:]
					if fullblock:
						self.commands = self.commands[:i] + self.commands[i+1 :]
					else:
						self.commands[i] = self.commands[i][:j] + self.commands[i][j+len(ar):]
					self.strip_command_last_found_index = i
					return ret_val
		
	def brol(self):
		
		global dirs_created
		for a in self.ar:
			if (not self.window) and self.check_window(a): break
			if not (self.window):
				if not self.check_frame(a):
					self.check_def(a)
			else:
				self.check_frame(a)
			
		if not self.window: return
		for a in self.pp:
			tfp = File.fp("tussenstap\\" + a)
			d = File.dir(tfp)
			tfp = d + File.name(a) + ".json"
			
			if not d in dirs_created:
				dirs_created[d] = 1
				Dir.create(d)
				sleep(1)
			if (not self.only_exists) or not File.exists(tfp):
				File.jwrite(tfp,dict(fp=self.fp,window=self.window,ar=self.ar,frames=self.frames,widgets=self.widgets))
			
		

		for f in self.frames.values():
			for x in f['els']:
				n = 0
				for a in x:
					if a == 'at':
						n += 1
				if n != 1:
					File.jwrite("out\\window.json",self.window)
					File.jwrite("out\\ar.json",self.ar)
					File.jwrite("out\\frames.json",self.frames)
					File.jwrite("out\\widgets.json",self.widgets)
					print(self.fp)
					print(x)
					exit()
					
		
			
	
	def check_frame(self,a):
		rest = self.same_ar(a,['define','frame'])
		if not rest: return
		frame_name = rest[0]

		if not frame_name in self.frames:
			self.frames[frame_name] = dict(name=frame_name,els=[],w=[])
		
		rest = rest[1:]
		el = []
		
		i = len(rest) - 1
		while i != -1:
			a = rest[i]
			if a == 'with':
				self.frames[frame_name]['w'].append(rest[i:])
				rest = rest[:i]
				break
			elif a in self.widgets:
				break
			i -= 1
		
		def is_decimal(x):
			try:
				s = float(x)
				return 1
			except:
				return 0
			
		def check_new_widget(x):
			if x in self.widgets: return 1
			if not x.startswith('"') and x.find('.') != -1 and (not is_decimal(x)): return 1
			return 0
		
		for i,x in enumerate(rest):
			if check_new_widget(x):
				if el:
					self.frames[frame_name]['els'].append(el)
					el = []
			elif x == 'view-as' and ((len(el) > 3) or (not el)):
				if el:
					self.frames[frame_name]['els'].append(el[:-1])
					el = el[-1:]
			el.append(x)
		if el:
			self.frames[frame_name]['els'].append(el)
		return 1
	
	def check_frame(self,a):
		rest = self.same_ar(a,['define','frame'])
		if not rest: return
		frame_name = rest[0]

		if not frame_name in self.frames:
			self.frames[frame_name] = dict(name=frame_name,els=[],w=[])
		
		rest = rest[1:]
		el = []
		
		i = len(rest) - 1
		while i != 0:
			a = rest[i]
			if a == 'with':
				self.frames[frame_name]['w'].append(rest[i:])
				rest = rest[:i]
				break
			elif a in self.widgets and (rest[i-1].find('button') == -1):
				break
			i -= 1
		
		def is_decimal(x):
			try:
				s = float(x)
				return 1
			except:
				return 0
			
		def check_new_widget(i,x):
			if x in self.widgets: return 1
			if not x.startswith('"') and (x.find('(') == -1) and x.find('.') != -1 and (not is_decimal(x)): return 1
			return 0
		
		for i,x in enumerate(rest):
			if check_new_widget(i,x):
				if el:
					self.frames[frame_name]['els'].append(el)
					el = []
			elif x == 'view-as' and rest[i+1] == 'text':
				if len(el) > 1:
					self.frames[frame_name]['els'].append(el[:-1])
				el = el[-1:]
			el.append(x)
		if el:
			self.frames[frame_name]['els'].append(el)
		return 1
	
	def check_def(self,a):
		rest = self.same_ar(a,['define'])
		if not rest: return
		found = 0
		for i,x in enumerate(rest):
			if x in define_types:
				rest[i] = define_types[x]
				found = rest[i:]
				break
				
		if not found: return
		self.widgets[found[1]] = found
		
	def has_ar(self,a,b):
		start_ind = -1
		for i,x in enumerate(a):
			if x == b[0]:
				if self.same_ar(a[i:],b):
					return 1
			
	def same_ar(self,a,b):
		try:
			for i,x in enumerate(b):
				if a[i] != x: return 0
			return a[len(b):]
		except:
			return 0
			
	def find_assign(self,a):
		for i,b in enumerate(a):
			if b == 'assign':
				t = {}
				n = i
				n += 1
				while (n + 2) < len(a):
					t[a[n]] = a[n+2]
					n += 3
				return t
			
	def check_window(self,a):
		rest = self.same_ar(a,"if,session:display-type,=".split(','))
		if not rest: return 0
		if not self.has_ar(rest,["create","window"]): return 0
		self.window = self.find_assign(rest)
		return 1
		

def run():

	last_failed = r'c:\develop\l_intex\intex\prg\syslogging.w'
	last_failed_found = 1
	#os.system('del tussenstap\\*.json /s/q/f')
	#ENV.wfiles = [File.fp(fp).lower() for fp in [r'C:\develop\l_intex\Intex\prg\browacoh.w']]
	for fp in ENV.wfiles:
		last_failed_found = last_failed_found or fp == last_failed
		if not last_failed_found: continue
		f = Ana(fp)
		#f = Ana(r'C:\develop\l_intex\Intex\prg\mainartk.w')
		


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

def get_version():
	with open("version.txt","r") as f:
		return f.read()
	

setuptools.setup(
    name="jivi",
    version=get_version(),
    author="Joris Vercleyen",
    author_email="jorisvercleyen@gmail.com",
    description="Joris Vercleyen his personal library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)